/*********************************************************
*Copyright (C), 2021, Shanghai Eastsoft Microelectronics Co., Ltd
*文件名:  main.c
*作  者:  AE Team
*版  本:  V1.00
*日  期:  2021/11/8
*描  述:  CRC演示程序
          给定数据，计算CRC值，与目标值进行对比
*备  注:  适用于ES8H018x
          本软件仅供学习和演示使用，对用户直接引用代码所带来的风险或后果不承担任何法律责任。
**********************************************************/
#include "main.h"
#include "string.h"
int test_crc(void)
{
    uint8_t i;
    uint8_t in_data[6] = {0x11, 0x22, 0x33, 0x44, 0x55, 0x66};
    uint32_t out_data = 0;
    CRC_CONValueStrutType x;
    memset(&x, 0, sizeof(x));
    x.xorout = CRC_XOROUT_NORMAL;                //数据输出不取反
    x.refout = CRC_REFOUT_REVERSE;               //数据输出倒序
    x.refin = CRC_REFIN_REVERSE;                 //数据输入倒序
    x.byte = CRC_BYTE_BYTE;                      //字节位宽
    x.mode = CRC_MOD_CRC16;                      //CRC16模型
    x.flash = DISABLE;                           //不访问Flash
    x.init_data_type = CRC_INIT_DATA_ALL_0;      //初始为全0
    x.hs_type = CRC_HS_TYPE_DISABLE;             //禁止高速模式
    CRC_Init(&x);

    for (i = 0; i < 6; i++)
    {
        CRC_WriteData(in_data[i]);
        CRC_Trig(CRC_TRIG_USER_VERIFY);  //用户校验模式
    }

    out_data = CRC_GetData();

    if (out_data == 0x6D0B)
        return 1;
    else
        return 0;
}

int main(void)
{
    SystemHRCSelect(SCU_HRC_16M);       //HRC选择16MHz
    SystemClockConfig();
    DeviceClockAllEnable();

    while (test_crc() == 0);

    while (1);
}





