/*********************************************************
*Copyright (C), 2021, Shanghai Eastsoft Microelectronics Co., Ltd.
*文件名:  main.c
*作  者:  AE Team
*版  本:  V1.00
*日  期:  2021/11/4
*描  述:  GPIO按键中断演示主程序
*         PA20每产生一次按键中断，PA12电平翻转一次
*备  注:  适用于ES8H018x
          本软件仅供学习和演示使用，对用户直接引用代码所带来的风险或后果不承担任何法律责任。
**********************************************************/
#include "main.h"
#include "string.h"
/***************************************************************
 函数名：void KINTx_Init(void)
 描  述：按键中断初始化函数
 输入值：无
 输出值：无
 返回值：无
***************************************************************/
void KINTx_Init(void)
{
    KINT_InitSettingType y;

    GPIO_InitStruType x;
    memset(&x, 0, sizeof(x));
    memset(&y, 0, sizeof(y));
    x.GPIO_Signal = GPIO_Pin_Signal_Digital;
    x.GPIO_Direction = GPIO_Dir_In;
    x.GPIO_Func = GPIO_Func_0;
    x.GPIO_PUEN = GPIO_PUE_Input_Disable;
    x.GPIO_PDEN = GPIO_PDE_Input_Enable;
    x.GPIO_OD = GPIO_ODE_Output_Disable;
    x.GPIO_DS = GPIO_DS_Output_Normal;
    GPIO_Init(GPIOA, GPIO_Pin_20, &x);

    y.IE_Set = KINT_IE_Set_Enable;
    y.Trigger_Style = KINT_Trigger_Rising_Edge;
    KINT_Init(GPIOA, GPIO_Pin_20, &y);
    NVIC_Init(NVIC_KINT_IRQn, NVIC_Priority_1, ENABLE);

    return;
}

int main(void)
{
    SystemHRCSelect(SCU_HRC_16M);       //HRC选择16MHz
    SystemClockConfig();                //配置时钟
    DeviceClockAllEnable();             //打开所有外设时钟

    LightInit();                        //指示灯初始化
    KINTx_Init();

    while (1)
    {

    }
}
