/*********************************************************
*Copyright (C), 2021, Shanghai Eastsoft Microelectronics Co., Ltd.
*文件名:  main.c
*作  者:  AE Team
*版  本:  V1.00
*日  期:  2021/11/4
*描  述:  FLASH IAP演示主程序
          FLASH的0xF000写数据0x12345678
          读取改地址数据是否写入，写入成功点灯LD1，失败点灯LD4
*备  注:  适用于ES8H018x
          本软件仅供学习和演示使用，对用户直接引用代码所带来的风险或后果不承担任何法律责任。
**********************************************************/
#include "main.h"

int main(void)
{
    uint32_t g_rbuf = 0;
    uint32_t data = 0x12345678;
    uint32_t sta1 = 0;
    uint32_t sta2 = 0;
    uint32_t sta3 = 0;
    uint32_t sta4 = 0;

    SystemHRCSelect(SCU_HRC_16M);       //HRC选择16MHz
    SystemClockConfig();                //配置时钟HRC
    DeviceClockAllEnable();             //打开所有外设时钟

    LightInit();

    GPIOB_SetBit(GPIO_Pin_15);   //熄灭LD4
    GPIOB_SetBit(GPIO_Pin_14);   //熄灭LD3
    GPIOA_SetBit(GPIO_Pin_12);   //熄灭LD2
    GPIOA_SetBit(GPIO_Pin_11);   //熄灭LD1

    IAP_FLASH_Unlock();

    sta1 = IAP_Code_PageErase(START_ADDR, ~START_ADDR);
    sta2 = IAPRead(&g_rbuf, START_ADDR, 1);
    sta3 = IAP_Code_WordProgram(START_ADDR, ~START_ADDR, data);
    sta4 = IAPRead(&g_rbuf, START_ADDR, 1);

    IAP_FLASH_Lock();

    while (1)
    {
        if (g_rbuf == 0x12345678 && sta1 == SUCCESS && sta2 == SUCCESS && sta3 == SUCCESS && sta4 == SUCCESS)
            GPIOA_ResetBit(GPIO_Pin_12);
        else
            GPIOB_ResetBit(GPIO_Pin_14);
    }
}

